<?php

namespace App\Repository;

use App\Entity\Article;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @extends ServiceEntityRepository<Article>
 */
class ArticleRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Article::class);
    }

    //    /**
    //     * @return Article[] Returns an array of Article objects
    //     */
    //    public function findByExampleField($value): array
    //    {
    //        return $this->createQueryBuilder('a')
    //            ->andWhere('a.exampleField = :val')
    //            ->setParameter('val', $value)
    //            ->orderBy('a.id', 'ASC')
    //            ->setMaxResults(10)
    //            ->getQuery()
    //            ->getResult()
    //        ;
    //    }

    //    public function findOneBySomeField($value): ?Article
    //    {
    //        return $this->createQueryBuilder('a')
    //            ->andWhere('a.exampleField = :val')
    //            ->setParameter('val', $value)
    //            ->getQuery()
    //            ->getOneOrNullResult()
    //        ;
    //    }

    public function findLastPublicationDate(): ?\DateTimeImmutable
    {
        return $this->createQueryBuilder('a')
            ->select('a.publicationDate')
            ->orderBy('a.publicationDate', 'DESC')
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult()['publicationDate'] ?? null
        ;
    }

    public function findByTag(string $tag, bool $onlyPublished = true): array
    {
        $qb = $this->createQueryBuilder('a');

        if ($onlyPublished) {
            $qb->andWhere('a.state = :published')
                ->setParameter('published', 'published')
            ;
        }

        return $qb
            ->innerJoin('a.tags', 't')
            ->andWhere($qb->expr()->like('t.name', ':tag'))
            ->setParameter('tag', '%'.$tag.'%')
            ->orderBy('a.publicationDate', 'DESC')
            ->getQuery()
            ->getResult()
        ;
    }
}
